/***************************************************************************
                          fitdlg.h  -  description                              
                             -------------------                                         
    begin                : Tue May 25 1999                                           
    copyright            : (C) 2000 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef FITDLG_H
#define FITDLG_H

#include <qdialog.h>
#include <qtableview.h>
#include <qlist.h>
#include "kplots.h"

#define SHOWDLG 1
#define AUTOFIT 2
#define SAVEPAR 4
#define FOLLOW  8

class KplDoc;
class QLineEdit;
class QLabel;
class QListBox;
class QSpinBox;
class QCheckBox;
class ArrayItem;
class FunItem;
class LMFit;

/**
  * Correlation matrix table class. Provides a table view for displaying
  * the parameter correlation matrix.
  * @author Werner Stille
  */
class CorrTable : public QTableView {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param np number of fitted parameters.
   * @param ip array with parameter indices.
   * @param corr np * np matrix with correlation coefficients.
   */
	CorrTable(QWidget *parent, int np, int *ip, double *corr);
  /** Destructor. */
  ~CorrTable();

protected:
  /**
   * Draws table cell text (parameter indices or correlation coefficients).
   * @param p pointer to QPainter object.
   * @param row row index.
   * @param col column index.
   */
	virtual void paintCell(QPainter *p, int row, int col);
  int np, *ip;
  double *corr;

protected slots:
  /** Sets the widgets background color to that of a window. */
  void slotSetBackground();
};

/**
  * Correlation matrix dialog class. Provides a dialog for displaying
  * the parameter correlation matrix.
  *@author Werner Stille
  */
class CorrDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param np number of fitted parameters.
   * @param ip array with parameter indices.
   * @param corr np * np matrix with correlation coefficients.
   */
	CorrDlg(QWidget *parent, int np, int *ip, double *corr);
  /** Destructor. */
  ~CorrDlg();

protected:
  /**
   * Overloaded for adjusting the size of the table.
   */
  virtual void resizeEvent(QResizeEvent *ev);
  CorrTable *table;
};

/**
  * Error model dialog class. Provides a dialog for setting
  * the calculation of the error bars for nonlinear fits.
  * @author Werner Stille
  */
class ErrModDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param a0 pointer to structure containing the Autoplot settings.
   */
	ErrModDlg(QWidget *parent, KplDoc *model, AutoStruct *a0);
  /** Destructor. */
	~ErrModDlg();

protected:
  KplDoc *m;
  AutoStruct *a, *at;
  QLineEdit *fileName, *func;

protected slots:
  /** Displays the dialog for selection of a library file to load. */
  void slotFile();
  /** Displays the dialog for selection of a function. */
  void slotFunc();
  /** Displays the dialog for selection of a parameter file to read. */
  void slotGetParameter();
  /** Displays the dialog for parameter editing. */
  void slotEditParameter();
  /** Displays the dialog for selection of a parameter file to save. */
  void slotSaveParameter();
  /** Accepts changes and leaves the dialog. */
  void slotOK();
  /** Displays help for the function settings dialog. */
  void slotHelp();
};

/**
  * Fit dialog class. Provides a dialog for performing a parameter fit
  * of an ordinary function to a data set of an array.
  *@author Werner Stille
  */
class FitDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param ad pointer to array data structure.
   * @param fd pointer to function data structure.
   * @param mode combination of SHOWDLG and AUTOFIT.
   */
	FitDlg(QWidget *parent, KplDoc *model, ArrayItem *ad, FunItem *fd, int mode);
  /** Destructor. */
	~FitDlg();

public slots:
  /** Performs Levenberg-Marquardt fit. */
  void slotFit();
  /** Applies settings without leaving the dialog (if displayed). */
  void slotApply();

protected:
  /**
   * Checks and copies settings and parameters.
   * @param accept true for leaving the dialog, false for applying.
   */
  void getValues(bool accept);
  /**
   * saves parameters and their errors to file.
   * @param url URL of the file.
   */
  void saveParameter(QString url);
  int ip[NPMAX], np, dlgMode;
  double p[NPMAX], pErr[NPMAX], pMod[NPMAX], corr[NPMAX * NPMAX];
  KplDoc *m;
  ArrayItem *ad;
  FunItem *fd;
  AutoStruct *aut;
  LMFit *lm;
  QListBox *results;
  QLineEdit *eTol, *par[NPMAX];
  QLabel *epar[NPMAX];
  QCheckBox *enFit[NPMAX], *errCol, *nonLin;
  QSpinBox *sItmax;
  QPushButton *showCorr;
  QPushButton *errMod;
  QList<QWidget> widgetList;

protected slots:
  /** Displays the dialog for selection of a parameter file to read. */
  void slotGetParameter();
  /** Displays the dialog for selection of a parameter file to save. */
  void slotSaveParameter();
  /**
   * Disables/enables error model button.
   * @param off disables button when true.
   */
  void disableErrMod(bool off);
  /**
   * Disables/enables widgets for nonlinear fits.
   * @param on enables widgets when true.
   */
  void enableNonLin(bool on);
  /** Accepts changes and leaves the dialog. */
  void slotOK();
  /** Stops fitting or leaves the dialog without applying the changes. */
  void slotCancel();
  /** Displays help for the function settings dialog. */
  void slotHelp();
  /** Displays the dialog for setting the calculation of error bars. */
  void slotErrMod();
  /**
   * Displays message line in listbox.
   * @param msg message text.
   */
  void slotMessage(const char *msg);
  /** Displays parameter correlation matrix. */
  void slotCorr();

signals:
  /**
    * Emitted when applied changes require repainting the
    * page view without leaving the dialog.
    */
  void modelChanged();
};

#endif
